'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const router_1 = require("./type/router");
const service_1 = require("./type/service");
const EggLoader = require('egg').AppWorkerLoader;
exports.EggLoader = EggLoader;
class AppWorkerLoader extends EggLoader {
    static registerServiceToAppIOC(app) {
        const ioc = app.iocContext;
        service_1.getServices().forEach(service => {
            const ServiceType = service.classConstructor;
            ioc.register(ServiceType, ServiceType, { autoNew: false });
        });
    }
    static registerRouter(app) {
        router_1.getRouters()
            .sort((a, b) => {
            if (a.url === b.url) {
                return 0;
            }
            if (a.url === '/*') {
                return 1;
            }
            return a.url > b.url ? -1 : 1;
        })
            .forEach(route => {
            // can't get params in url, when url is array. (chair? egg? koa?)
            [].concat(typeof route.url === 'function' ? route.url(app) : route.url)
                .forEach(url => app.register(url, [].concat(route.method || 'all'), [].concat(...route.beforeMiddleware.map(m => m(app)), route.call(), ...route.afterMiddleware.map(m => m(app)))));
        });
    }
    /**
     * 开始加载所有约定目录
     * @since 1.0.0
     */
    load() {
        super.load();
        // load app files
        this.loadApp();
        // register service
        AppWorkerLoader.registerServiceToAppIOC(this.app);
        // register router
        AppWorkerLoader.registerRouter(this.app);
    }
    loadDir(dirPath) {
        fs.readdirSync(dirPath)
            .filter(dir => [
            'assets',
            'view',
            'template',
            'public',
            'test',
        ].indexOf(dir) < 0)
            .forEach(dirName => {
            const fullPath = path.join(dirPath, dirName);
            const stat = fs.statSync(fullPath);
            if (stat.isDirectory()) {
                this.loadDir(fullPath);
            }
            else if (stat.isFile && path.parse(dirName).ext === '.js') {
                try {
                    require(fullPath);
                }
                catch (err) {
                    err.message = `[egg-typed] load file: ${fullPath}, error: ${err.message}`;
                    throw err;
                }
            }
        });
    }
    loadApp() {
        const self = this;
        const baseDir = self.options.baseDir;
        this.loadDir(path.join(baseDir, 'app'));
    }
}
exports.default = AppWorkerLoader;
